import "package:dart_irc/irc.dart";
import "package:dart_irc/model.dart";

abstract class CommandProcessor {
  bool applies(Connection connection, String command, String message);
  void process(Connection connection, List<String> parts, String message);
}

class Join extends CommandProcessor {
  @override
  bool applies(Connection connection, String command, String message) {
    return command.toLowerCase() == "join";
  }

  @override
  void process(Connection connection, List<String> parts, String message) {
    String source = parts[0];
    int index = source.indexOf("!");
    String nickname = source.substring(1, index);
    String username = source.substring(index + 1, source.indexOf("@"));
    Channel channel = new Channel(parts[2]);
    User user = new User(nickname, username);
    connection.callback.onJoin(connection, user, channel);
  }
}

class Quit extends CommandProcessor {
  @override
  bool applies(Connection connection, String command, String message) {
    return command.toLowerCase() == "quit";
  }

  @override
  void process(Connection connection, List<String> parts, String message) {
    String source = parts[0];
    int index = source.indexOf("!");
    String nickname = source.substring(1, index);
    String username = source.substring(index + 1, source.indexOf("@"));
    String reason = message.substring(message.indexOf("QUIT") + 6);
    User user = new User(nickname, username);
    connection.callback.onQuit(connection, user, reason);
  }
}