import "package:dart_irc/model.dart";
import "package:dart_irc/callback.dart";
import "package:dart_irc/command.dart";

import "dart:io";
import "dart:async";

List<CommandProcessor> processors = [ new Join(), new Quit() ];

class Connection {
  User user;
  Server server;
  Callback callback;
  Socket socket;

  Connection(this.user, this.server, this.callback);

  Future<dynamic> connect() async {
    socket = await Socket.connect(server.host, server.port);
    nick(user.nickname);
    _send("USER ${user.username} ${user.visible ? 0 : 8} * : ${user.realname}");
    callback.onConnect(this);
    socket.listen((raw) {
      String message = new String.fromCharCodes(raw);
      List<String> parts = message.split(" ");
      callback.onMessage(this, message);
      processors.where((it) => it.applies(this, parts[1], message)).forEach((it) => it.process(this, parts, message));
    });
  }

  void disconnect() {
    callback.onDisconnect(this);
    socket.destroy();
    socket = null;
  }

  void nick(String nickname) {
    _send("NICK ${user.nickname}");
  }

  void join(var target) {
    Channel channel;
    if (target is String)
      channel = new Channel(target);
    else if (target is Channel)
      channel = target;
    else
      throw "function overloading when?";
    if (!channel.name.startsWith("#"))
      channel.name = "#${channel.name}";
    _send("JOIN :${channel.name}");
  }

  void pong(String ping) {
    _send("PONG ${ping.substring(ping.indexOf(':'))}");
  }

  void _send(String message) {
    if (socket == null)
      throw "Cannot write to socket: socket is null";
    if (message.length > 510)
      throw new ArgumentError("Message length cannot be greater than 510");
    print("[out] ${message}");
    socket.write("${message}\r\n");
  }
}